#import "SharetraceFlutterPlugin.h"
#import <SharetraceSDK/SharetraceSDK.h>

@implementation SharetraceFlutterPlugin

static NSString * const key_code = @"code";
static NSString * const key_msg = @"msg";
static NSString * const key_data = @"data";
static NSString * const key_paramsData = @"paramsData";
static NSString * const key_resumePage = @"resumePage";

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    [Sharetrace start];
    FlutterMethodChannel* channel = [FlutterMethodChannel
                                     methodChannelWithName:@"sharetrace_flutter_plugin"
                                     binaryMessenger:[registrar messenger]];
    SharetraceFlutterPlugin* instance = [[SharetraceFlutterPlugin alloc] init];
    instance.flutterMethodChannel = channel;
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"getInstallTrace" isEqualToString:call.method]) {
        [Sharetrace getInstallTrace:^(AppData * _Nullable appData) {
            if (appData == nil) {
                NSDictionary *ret = [SharetraceFlutterPlugin parseToResultDict:-1 :@"Extract data fail." :@"" :@""];
                [self response:ret];
                return;
            }
            
            NSDictionary *ret = [SharetraceFlutterPlugin parseToResultDict:200 :@"Success" :appData.paramsData :appData.resumePage];
            [self response:ret];
            
        } :^(NSInteger code, NSString * _Nonnull msg) {
            NSDictionary *ret = [SharetraceFlutterPlugin parseToResultDict:code :msg :@"" :@""];
            [self response:ret];
        }];
        
    } else {
        result(FlutterMethodNotImplemented);
    }
}

- (void)response:(NSDictionary *) ret {
    [self.flutterMethodChannel invokeMethod:@"onInstallResponse" arguments:ret];
}

+ (NSDictionary*)parseToResultDict:(NSInteger)code :(NSString*)msg :(NSString*)paramsData :(NSString*)resumePage {
    NSMutableDictionary* dict = [[NSMutableDictionary alloc] init];

    dict[key_code] = [[NSNumber numberWithInteger:code] stringValue];
    dict[key_msg] = msg;
    dict[key_paramsData] = paramsData;
    dict[key_resumePage] = resumePage;

    return dict;
}

@end
